library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity effector_avnet is
port (
	signal avs_s1_clk : in std_logic;
	signal avs_s1_reset_n : in std_logic;
	signal avs_s1_chipselect : in std_logic;
	signal avs_s1_write : in std_logic;
	signal avs_s1_address : in std_logic_vector( 7 downto 0);
	signal avs_s1_writedata : in std_logic_vector(15 downto 0);
	signal avs_s1_byteenable : in std_logic_vector(1 downto 0);	
	
	AUD_ADCLRCK  : out std_logic;   --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in  std_logic;   --    Audio CODEC ADC Data
    AUD_BCLK     : inout std_logic;  --    Audio CODEC Bit-Stream Clock
    AUD_DACLRCK  : out std_logic;   --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out std_logic;   --    Audio CODEC DAC Data
	I2C_SCLK	 : out std_logic;
	I2C_SDAT	 : inout std_logic;
	audio_clk	 : in std_logic
);
end effector_avnet;

architecture rtl of effector_avnet is

component effector is
port (

	clk : in std_logic;
	audio_clk : in std_logic;
	reset_n : in std_logic;

	-- for temporal testing... we will include this as bus commande from CPU!
	Ldistortion_enable : in std_logic;
	Lclip : in std_logic_vector(15 downto 0);	

	Rdistortion_enable : in std_logic;
	Rclip : in std_logic_vector(15 downto 0);	

    Lvibrato_enable: in std_logic;
    Lvibrato_amp: in std_logic_vector(2 downto 0);
	Lvibrato_freq: in std_logic_vector(3 downto 0);

    Rvibrato_enable: in std_logic;
    Rvibrato_amp: in std_logic_vector(2 downto 0);
	Rvibrato_freq: in std_logic_vector(3 downto 0);
	
	Lchorus_enable: in std_logic;
	Lchorus_amp: in std_logic_vector(2 downto 0);
	Lchorus_freq: in std_logic_vector(3 downto 0);
	Lchorus_mix_delay: in std_logic_vector(3 downto 0);

	Rchorus_enable: in std_logic;
	Rchorus_amp: in std_logic_vector(2 downto 0);
	Rchorus_freq: in std_logic_vector(3 downto 0);
	Rchorus_mix_delay: in std_logic_vector(3 downto 0);
	
	Lvol_valid: in std_logic;
	Lvol_data: in std_logic_vector(6 downto 0);

	Rvol_valid: in std_logic;
	Rvol_data: in std_logic_vector(6 downto 0);

    -- Audio interface signals
	AUD_ADCLRCK  : out std_logic;   --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in  std_logic;   --    Audio CODEC ADC Data
    AUD_BCLK     : inout std_logic;  --    Audio CODEC Bit-Stream Clock
    AUD_DACLRCK  : out std_logic;   --    Audio CODEC DAC LR Clock
    AUD_DACDAT   : out std_logic;   --    Audio CODEC DAC Data
	I2C_SCLK	 : out std_logic;
	I2C_SDAT	 : inout std_logic
);
end component;

	signal Ldistortion_enable : std_logic := '0';
	signal Lclip : std_logic_vector(15 downto 0) := "0000000010000000";	

	signal Rdistortion_enable :  std_logic := '0';
	signal Rclip :  std_logic_vector(15 downto 0) := "0000000010000000";	

    signal Lvibrato_enable:  std_logic := '0';
    signal Lvibrato_amp:  std_logic_vector(2 downto 0) := "100";
	signal Lvibrato_freq:  std_logic_vector(3 downto 0) := "0011";

    signal Rvibrato_enable:  std_logic := '0';
    signal Rvibrato_amp: std_logic_vector(2 downto 0) := "100";
	signal Rvibrato_freq:  std_logic_vector(3 downto 0) := "0011";
	
	signal Lchorus_enable:  std_logic := '0';
	signal Lchorus_amp:  std_logic_vector(2 downto 0) := "100";
	signal Lchorus_freq:  std_logic_vector(3 downto 0) := "0011";
	signal Lchorus_mix_delay:  std_logic_vector(3 downto 0) := X"7";

	signal Rchorus_enable:  std_logic := '0';
	signal Rchorus_amp:  std_logic_vector(2 downto 0) := "100";
	signal Rchorus_freq: std_logic_vector(3 downto 0) := "0011";
	signal Rchorus_mix_delay:  std_logic_vector(3 downto 0) := X"7";
	
	signal Lvol_valid:  std_logic := '0';
	signal Lvol_data:  std_logic_vector(6 downto 0) := "0000000";

	signal Rvol_valid:  std_logic := '0';
	signal Rvol_data:  std_logic_vector(6 downto 0) := "0000000";

	--Addresses:
	constant 	LVOL : std_logic_vector (7 downto 0) := X"00";
	constant	RVOL : std_logic_vector (7 downto 0) := X"02"; 
	constant	L_D_EN : std_logic_vector (7 downto 0) := X"04";
	constant	L_D_CLIP : std_logic_vector (7 downto 0) := X"06"; 
	constant	R_D_EN : std_logic_vector (7 downto 0) := X"08";
	constant	R_D_CLIP : std_logic_vector (7 downto 0) := X"0A"; 
				
	constant	L_V_EN : std_logic_vector (7 downto 0) := X"0C";
	constant	L_V_AMP : std_logic_vector (7 downto 0) := X"0E";
	constant	L_V_FREQ : std_logic_vector (7 downto 0) := X"10"; 
	constant	R_V_EN : std_logic_vector (7 downto 0) := X"12";
	constant	R_V_AMP : std_logic_vector (7 downto 0) := X"14";
	constant	R_V_FREQ : std_logic_vector (7 downto 0) := X"16"; 
				
	constant	L_C_EN : std_logic_vector (7 downto 0) := X"18";
	constant	L_C_AMP : std_logic_vector (7 downto 0) := X"1A";
	constant	L_C_FREQ : std_logic_vector (7 downto 0) := X"1C";
	constant	L_C_MIX : std_logic_vector (7 downto 0):= X"1E";
	constant	R_C_EN : std_logic_vector (7 downto 0) := X"20";
	constant	R_C_AMP : std_logic_vector (7 downto 0) := X"22";
	constant	R_C_FREQ : std_logic_vector (7 downto 0) := X"24";
	constant	R_C_MIX : std_logic_vector (7 downto 0):= X"26";
	
begin




U1: effector port map (
	clk => avs_s1_clk,
	audio_clk => audio_clk,
	reset_n => avs_s1_reset_n,

	-- for temporal testing... we will include this as bus commande from CPU!
	Ldistortion_enable => Ldistortion_enable,
	Lclip => Lclip,
	
	Rdistortion_enable =>Rdistortion_enable,
	Rclip =>Rclip,

    Lvibrato_enable=>Lvibrato_enable,
    Lvibrato_amp=>Lvibrato_amp,
	Lvibrato_freq=>Lvibrato_freq,
	
    Rvibrato_enable=>Rvibrato_enable,
    Rvibrato_amp=>Rvibrato_amp,
	Rvibrato_freq=>Rvibrato_freq,
	
	Lchorus_enable=>Lchorus_enable,
	Lchorus_amp=>Lchorus_amp,
	Lchorus_freq=>Lchorus_freq,
	Lchorus_mix_delay=>Lchorus_mix_delay,

	Rchorus_enable=>Rchorus_enable,
	Rchorus_amp=>Rchorus_amp,
	Rchorus_freq=>Rchorus_freq,
	Rchorus_mix_delay=>Rchorus_mix_delay,
	
	Lvol_valid=>Lvol_valid,
	Lvol_data=>Lvol_data,

	Rvol_valid=>Rvol_valid,
	Rvol_data=>Rvol_data,

    -- Audio interface signals
	AUD_ADCLRCK  => AUD_ADCLRCK, --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   => AUD_ADCDAT,  --    Audio CODEC ADC Data
    AUD_BCLK     => AUD_BCLK,     --    Audio CODEC Bit-Stream Clock
    AUD_DACLRCK  => AUD_DACLRCK,  --    Audio CODEC DAC LR Clock
    AUD_DACDAT   => AUD_DACDAT,   --    Audio CODEC DAC Data
	I2C_SCLK	 => I2C_SCLK,
	I2C_SDAT	 => I2C_SDAT

);

	
--	signal Lvol_valid: in std_logic;
--	signal Lvol_data: in std_logic_vector(6 downto 0);

--	signal Rvol_valid: in std_logic;
--	signal Rvol_data: in std_logic_vector(6 downto 0);

--	signal Ldistortion_enable : in std_logic;
--	signal Lclip : in std_logic_vector(15 downto 0);	

--	signal Rdistortion_enable : in std_logic;
--	signal Rclip : in std_logic_vector(15 downto 0);	

--  signal Lvibrato_enable: in std_logic;
--  signal Lvibrato_amp: in std_logic_vector(2 downto 0);
--	signal Lvibrato_freq: in std_logic_vector(3 downto 0);

--  signal Rvibrato_enable: in std_logic;
--  signal Rvibrato_amp: in std_logic_vector(2 downto 0);
--	signal Rvibrato_freq: in std_logic_vector(3 downto 0);
	
--	signal Lchorus_enable: in std_logic;
--	signal Lchorus_amp: in std_logic_vector(2 downto 0);
--	signal Lchorus_freq: in std_logic_vector(3 downto 0);
--	signal Lchorus_mix_delay: in std_logic_vector(3 downto 0);

--	signal Rchorus_enable: in std_logic;
--	signal Rchorus_amp: in std_logic_vector(2 downto 0);
--	signal Rchorus_freq: in std_logic_vector(3 downto 0);
--	signal Rchorus_mix_delay: in std_logic_vector(3 downto 0);


process(avs_s1_clk)
begin
	if (avs_s1_clk 'event and avs_s1_clk = '1') then
		if(avs_s1_reset_n = '0') then
			Lvol_valid <='0';
			Lvol_data <= "0000000";

			Rvol_valid <='0';
			Rvol_data <= "0000000";

			Ldistortion_enable <='0';
			Lclip  <= "0000000010000000";

			Rdistortion_enable  <='0';
			Rclip  <= "0000000010000000";

			Lvibrato_enable <= '0';
			Lvibrato_amp <= "100";
			Lvibrato_freq <= "0011";

			Rvibrato_enable <= '0';
			Rvibrato_amp <= "100";
			Rvibrato_freq <= "0011";
	
			Lchorus_enable <= '0';
			Lchorus_amp <= "100";
			Lchorus_freq <= "0011";
			Lchorus_mix_delay <= X"7";

			Rchorus_enable <= '0';
			Rchorus_amp <= "100";
			Rchorus_freq <= "0011";
			Rchorus_mix_delay <= X"7";			
		else
			if(avs_s1_chipselect = '1') then
				case (avs_s1_address) is
				when LVOL => 
					Lvol_valid <='1';
					Lvol_data <= avs_s1_writedata(6 downto 0);
				when RVOL => 
					Rvol_valid <='1';
					Rvol_data <= avs_s1_writedata(6 downto 0);				
				when L_D_EN =>
					Ldistortion_enable <= avs_s1_writedata(0);
				when L_D_CLIP => 
					Lclip <= avs_s1_writedata;
				when R_D_EN =>
					Rdistortion_enable <= avs_s1_writedata(0);
				when R_D_CLIP =>
					Rclip <= avs_s1_writedata;
				when L_V_EN =>
					Lvibrato_enable <= avs_s1_writedata(0);
				when L_V_AMP =>
					Lvibrato_amp <= avs_s1_writedata(2 downto 0);
				when L_V_FREQ =>
					Lvibrato_freq <= avs_s1_writedata(3 downto 0);
				when R_V_EN =>
					Rvibrato_enable <= avs_s1_writedata(0);
				when R_V_AMP =>
					Rvibrato_amp <= avs_s1_writedata(2 downto 0);
				when R_V_FREQ =>
					Rvibrato_freq <= avs_s1_writedata(3 downto 0);
				when L_C_EN =>
					Lchorus_enable <= avs_s1_writedata(0);
				when L_C_AMP =>
					Lchorus_amp <= avs_s1_writedata(2 downto 0);
				when L_C_FREQ =>
					Lchorus_freq <= avs_s1_writedata(3 downto 0);
				when L_C_MIX =>
					Lchorus_mix_delay <= avs_s1_writedata(3 downto 0);
				when R_C_EN =>
					Rchorus_enable <= avs_s1_writedata(0);
				when R_C_AMP =>
					Rchorus_amp <= avs_s1_writedata(2 downto 0);
				when R_C_FREQ =>
					Rchorus_freq <= avs_s1_writedata(3 downto 0);
				when R_C_MIX =>
					Rchorus_mix_delay <= avs_s1_writedata(3 downto 0);
				when others =>
					Rvol_valid <='0';
					Lvol_valid <='0';
				end case;
			else
				--(avs_s1_chipselect = '0')
					Rvol_valid <='0';
					Lvol_valid <='0';		
			end if;
		end if;	
	end if;
end process;

end architecture;