library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity de2_wm8731_audio_in is
port (
    clk : in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
    reset_n : in std_logic;
    data_out : out std_logic_vector(15 downto 0);
    audio_req : out std_logic;

    -- Audio interface signals
	AUD_ADCLRCK  : out std_logic;   --    Audio CODEC ADC LR Clock
    AUD_ADCDAT   : in  std_logic;   --    Audio CODEC ADC Data
    AUD_BCLK     : inout std_logic  --    Audio CODEC Bit-Stream Clock
  );
end  de2_wm8731_audio_in;


architecture Behavioral of  de2_wm8731_audio_in is     

    signal lrck : std_logic;
    signal bclk : std_logic;
    signal xck : std_logic;
    
    signal lrck_divider : std_logic_vector (7 downto 0); 
    signal bclk_divider : std_logic_vector (3 downto 0);
    
    signal set_bclk : std_logic;
    signal set_lrck : std_logic;
    signal lrck_lat : std_logic;
	signal clr_bclk : std_logic;
    signal datain : std_logic;

    signal shift_in : std_logic_vector ( 15 downto 0);   
	signal shift_counter : integer := 15;
	
begin
    -- LRCK divider 
    -- Audio chip main clock is 18.432MHz / Sample rate 48KHz
    -- Divider is 18.432 MHz / 48KHz = 192 (X"C0")
    -- Left justify mode set by I2C controller
    
    process(clk, reset_n)
    begin
        if ( reset_n = '0' ) then 
            lrck_divider <= (others => '0');
        elsif ( clk'event and clk='1' ) then
            if ( lrck_divider = X"BF")  then        -- "C0" minus 1
                lrck_divider <= X"00";
            else 
                lrck_divider <= lrck_divider + '1';
            end if;
        end if;
    end process;

    process(clk, reset_n)
    begin
        if ( reset_n = '0' ) then 
            bclk_divider <= (others => '0');
        elsif ( clk'event and clk='1' ) then
            if ( bclk_divider = X"B" or set_lrck = '1')  then  
                bclk_divider <= X"0";
            else 
                bclk_divider <= bclk_divider + '1';
            end if;
        end if;
    end process;
    
  
    process ( lrck_divider ) 
    begin
        if ( lrck_divider = X"BF") then
            set_lrck <= '1';
        else
            set_lrck <= '0';
        end if;
    end process;
    
    process ( clk, reset_n) 
    begin
        if ( reset_n = '0') then
            lrck <= '0';
        elsif ( clk 'event and clk = '1') then 
            if ( set_lrck = '1') then 
                lrck <= not lrck;
            end if;
        end if;
    end process;
    
    -- BCLK divider    
    process ( bclk_divider ) 
    begin
        if ( bclk_divider (3 downto 0) = "0101") then
            set_bclk <= '1';                      
        else
            set_bclk <= '0';
        end if;      

        if ( bclk_divider (3 downto 0) = "1011") then
            clr_bclk <= '1';                      
        else
            clr_bclk <= '0';
        end if;              
    end process;
    
    process ( clk, reset_n) 
    begin
        if ( reset_n = '0') then
            bclk <= '0';
        elsif ( clk 'event and clk = '1') then 
            if ( set_lrck = '1' or clr_bclk = '1') then
                bclk <= '0';
            elsif ( set_bclk = '1') then 
                bclk <= '1';
            end if;
        end if;
    end process;

	process (clk)
	begin
	if ( clk 'event and clk = '1') then
--		if ( clr_bclk = '1') then 
        	-- shift_in <= shift_in (14 downto 0) & '0';
	--	els
		if (set_bclk = '1') then
	-- datain <= AUD_ADCDAT;
			shift_in(shift_counter) <= AUD_ADCDAT;
			if (shift_counter = 0) then
				shift_counter <= 15;
			else
				shift_counter <= shift_counter - 1;
			end if;
		end if;
	end if;
	end process;
	process(clk)
    begin
        if ( clk'event and clk='1' ) then
            lrck_lat <= lrck;
        end if;
    end process;
	process (clk) 
    begin
        if ( clk'event and clk = '1') then 
            if (( lrck_lat = '1' and lrck = '0') or ( lrck_lat = '0' and lrck = '1')) then
                audio_req <= '1';
            else
                audio_req <= '0';
            end if;
        end if;
    end process;
    -- Audio data shift output
    process ( clk, reset_n) 
    begin
        if ( clk 'event and clk = '1') then 
            if ( set_lrck = '1') then
               data_out <= shift_in;
            end if;
        end if;
    end process;

    -- Audio outputs
    
    AUD_BCLK     <= bclk; 
	AUD_ADCLRCK	 <= lrck;             

end architecture;


